/************************************************************************
*                                                                       *
*   The program of erasin elements  (Batch mode)                        *
*                                                                       *
*       type_opt:  Type Option (0:ALL, 100:POINT)                       *
*       sho_opt :  Sho Option  (0:NO-SHO, 2:ALL)                        *
*       vol     :  Volume                                               *
*       group   :  Group                                                *
*       user    :  User                                                 *
*       pnfn    :  DwgID                                                *
*                                                                       *
************************************************************************/
#include    <stdlib.h>
#include    <malloc.h>
#include    <memory.h>
#define     VOL_4                       /* Drive is 4 characters */
#include    "accfnc.h"

extern short logfopen(char*);
extern short loghed(char*,char*,char*,char*);
extern short logmsg(long);
extern short logdwgid(long);
extern void  logfclse(void);

static short erase(long,long);
static short erase_elements(long,long);

#define SPACE_CHAR            ' '
#define WILD_CHAR             '?'

#define VOL_LEN                 4
#define GROUP_LEN               4
#define USER_LEN                6
#define DWGID_LEN              20
#define LST_LEN                16

#define MODEL_TYPE_V3           3
#define PERMISSION_RW           1
#define PERMISSION_RO           2
#define PERMISSION_PW           3
#define PERMISSION_LK           4
#define PERMISSION_SC           5

#define IDATA_OFFSET_MODEL_TYPE 0
#define IDATA_OFFSET_PERMISSION 2
#define IDATA_OFFSET_3DELM      4

#define MODEL_NO                1

#define TYPE_ALL                0
#define TYPE_POINT            100
#define TYPE_PIVOT            110
#define TYPE_LAVER            120
#define TYPE_PIVOTLAVER       130
#define SHO_ALL                 2
#define SHO_NOSHO               0
#define TYPE_OFFSPLINE        510

short erselm
(
long    type_opt,
long    sho_opt,
char    *vol,
char    *group,
char    *user,
char    *dwgid
)
{
    short   s_retc, retc;           /* Return Code */
    long    idata[5];               /* Drawing Informations */
    long    savemod;
    long    compmod;

    /*--------------------------------------*/
    /* Query the compress mode of drawing   */
    /*--------------------------------------*/
    MC_flcomp(2L, &savemod);

    /*--------------------------------------*/
    /* Output the header message            */
    /*--------------------------------------*/
    loghed(vol, group, user, dwgid);
    retc = 0;

    /*------------------*/
    /* Set Drive        */
    /*------------------*/
    s_retc =MC_budriv(1L, 2L, vol, 10L);
    if (s_retc != 0) goto ERR_END;

    /*------------------------------*/
    /* Check the drawing model type */
    /*------------------------------*/
    s_retc = MC_dtyp( group, user, dwgid, idata );
    if (s_retc != 0) goto ERR_END;

    if (idata[IDATA_OFFSET_MODEL_TYPE] != MODEL_TYPE_V3) {
        s_retc = 1001L;
        goto ERR_END;
    }
    /*------------------------------*/
    /* Check the model permission   */
    /*------------------------------*/
    if ( idata[IDATA_OFFSET_PERMISSION] != PERMISSION_RW
      && idata[IDATA_OFFSET_PERMISSION] != PERMISSION_SC
       )
    {
        switch(idata[IDATA_OFFSET_PERMISSION]) {
        case PERMISSION_RO:
             s_retc = 20L;
            break;
        case PERMISSION_PW:
             s_retc = 23L;
            break;
        case PERMISSION_LK:
             s_retc = 1002L;
            break;
        default:
            break;
        } /* end of switch */
        goto ERR_END;
    }
    /*--------------------------------------*/
    /* Check the drawing including 3D model */
    /*--------------------------------------*/
    if (idata[IDATA_OFFSET_3DELM] != 1L) {
        s_retc = 21L;
        goto ERR_END;
    }
    /*--------------------------*/
    /* Open the model           */
    /*--------------------------*/
    s_retc = MC_buopen(2L, MODEL_NO, group, user, dwgid, 10L);
    if (s_retc != 0) {
        goto ERR_END;
    }
    /*------------------------------------*/
    /* The process of erasing elements    */
    /*------------------------------------*/
    s_retc = erase(type_opt, sho_opt);
    if (s_retc != 0) {
        MC_mdlclos(MODEL_NO);
        goto ERR_END;
    }
    /*----------------------------------------------*/
    /* Set the compress mode and file the drawing   */
    /*----------------------------------------------*/
    MC_flcompst(vol, group, user, dwgid, &compmod);
    MC_flcomp(1L, &compmod);
    s_retc = MC_bufile(5L, MODEL_NO, group, user, dwgid, 10L);
    if (s_retc != 0) {
        MC_mdlclos(MODEL_NO);
        goto ERR_END;
    }
    /*----------------------*/
    /* Close the model      */
    /*----------------------*/
    MC_mdlclos(MODEL_NO);
    
    /*------------------------------*/
    /* Restore the compress mode    */
    /*------------------------------*/
    MC_flcomp(1L, &savemod);
    logdwgid( (long)s_retc );
    return(0);

ERR_END:
    logdwgid( (long)s_retc );
    return(1);
}

/************************************************************************
*                                                                       *
*   Erase Elements in All View and Detail                               *
*                                                                       *
*       type_opt :  Type Option (0:ALL, 100:POINT)                      *
*       sho_opt  :  Sho Option  (0:NO-SHO, 2:ALL)                       *
*                                                                       *
************************************************************************/
static short erase(long type_opt, long sho_opt)
{
    short   ret;
    long    ii;
    long    ivdtln;
    long    ivudet;
    long    save_ivudet;
    long    number;
    long    iopt;
    long    numopt;

    /* Save Current View */
    iopt = 2L;
    numopt = 1L;
    ret = MC_bucds(iopt, MODEL_NO, numopt, &save_ivudet, &number, 10L);
    if(ret != 0) goto ERR;

    /* Get Number of View */
    iopt = 1L;
    ret = MC_gvdnum(iopt, MODEL_NO, &ivdtln);
    if(ret != 0) goto ERR;
    else if(ivdtln == 0L) return(0);

    /* Erase Elements in View */
    for(ii=0; ii<ivdtln; ii++)
    {
        /* Change Current View */
        iopt = 1L;
        ivudet = ii;
        ret = MC_bucds(iopt, MODEL_NO, numopt, &ivudet, &number, 10L);
        if(ret != 0) goto ERR;

        /* Erase Elements in View */
        ret = erase_elements( type_opt, sho_opt);
        if(ret != 0) goto ERR;
    }

    /* Get Number of Detail */
    iopt = 2L;
    ret = MC_gvdnum(iopt, MODEL_NO, &ivdtln);
    if(ret != 0) goto ERR;
    else if(ivdtln == 0L) return(0);

    /* Erase Elements in Detail */
    for(ii=0; ii<ivdtln; ii++)
    {
        /* Change Current Detail */
        iopt = 1L;
        ivudet = -(ii+1);
        ret = MC_bucds(iopt, MODEL_NO, numopt, &ivudet, &number, 10L);
        if(ret != 0) goto ERR;

        /* Erase Elements in Detail */
        ret = erase_elements( type_opt, sho_opt);
        if(ret != 0) goto ERR;
    }

    /* Reset Current View */
    iopt = 1L;
    ret = MC_bucds(iopt, MODEL_NO, numopt, &save_ivudet, &number, 10L);
    if(ret != 0) goto ERR;

    ret = 0L;
ERR:
    return(ret);
}

/************************************************************************
*                                                                       *
*   Erase Elements                                                      *
*                                                                       *
*       type_opt :  Type Option (0:ALL, 100:POINT)                      *
*       sho_opt  :  Sho Option  (0:NO-SHO, 2:ALL)                       *
*                                                                       *
************************************************************************/
static short erase_elements(long type_opt, long sho_opt)
{
    short   ret;
    short   s_ret;
    long    iopt;
    long    itype;
    long    istart;
    long    mptr;
    long    nxtptr;
    long    mptrar[2];
    long    iatno_dum = 0L;
    long    matr_dum;
    long    isho;

    s_ret = 0;

    /*----------------------------------*/
    /* Erase Offset Spline              */
    /*----------------------------------*/
    if( type_opt == TYPE_ALL ) {
        istart = 1;
        itype = TYPE_OFFSPLINE;
        for(; ;)
        {
            iopt = 1L;      /* Search Offset Spline */
            ret = MC_bufnd1(iopt, MODEL_NO, istart, &itype, iatno_dum, &mptr, &matr_dum, 10L);
            if(ret != 0) break;

            iopt = 1L;      /* Search Next Element */
            ret = MC_bunext(iopt, MODEL_NO, mptr, &istart, 10L);

                            /* Check Sho/No-Sho */
            isho = MC_bisho( MODEL_NO, mptr );
            if( isho != 0L && isho != 1L ) continue;
            if( sho_opt == SHO_NOSHO && isho == 1L ) continue;

                            /* Check Element Type */
            if( type_opt == TYPE_POINT ) continue;

            iopt = 1L;      /* Erase Element */
            mptrar[0] = mptr;
            ret = MC_budel(iopt, MODEL_NO, mptrar, 10L);
        }
    }

    /*----------------------*/
    /* Erase Other Elements */
    /*----------------------*/
    iopt = 1L;      /* Search Next Element */
    mptr = 1L;
    ret = MC_bunext(iopt, MODEL_NO, mptr, &nxtptr, 10L);
    mptr = nxtptr;
    if(ret == 0)
    {
        for(; ;)
        {
            iopt = 1L;      /* Search Next Element */
            ret = MC_bunext(iopt, MODEL_NO, mptr, &nxtptr, 10L);

                            /* Check Sho/No-Sho */
            isho = MC_bisho( MODEL_NO, mptr );
            if( isho != 0L && isho != 1L ) goto NEXT_ELM;
            if( sho_opt == SHO_NOSHO && isho == 1L ) goto NEXT_ELM;

                            /* Check Element Type */
            itype = MC_biityp( MODEL_NO, mptr );
            if( itype < 0L ) goto NEXT_ELM;
            if( type_opt == TYPE_POINT && itype != TYPE_POINT ) goto NEXT_ELM;
            if( itype == TYPE_PIVOT || itype == TYPE_LAVER || itype == TYPE_PIVOTLAVER ) goto NEXT_ELM;

            iopt = 1L;      /* Erase Element */
            mptrar[0] = mptr;
            MC_budel(iopt, MODEL_NO, mptrar, 10L);

NEXT_ELM:
            if(ret != 0) break; /* No Rest */
            mptr = nxtptr;
        }
    }

    return(s_ret);
}
