/************************************************************************
*                                                                       *
*       The program of erasing elements message log                     *
*                                                                       *
************************************************************************/
#include <stdio.h>
#include <string.h>
#include <direct.h>
#include <time.h>
#include <windows.h>

#define SPACE_CHAR            ' '

#define VOL_LEN                 4
#define GROUP_LEN               4
#define USER_LEN                6
#define DWGID_LEN              20
#define MSG_STRING_MAX        100

#define INVALID_PARA                      1
#define INVALID_PARA_FROM                11
#define INVALID_PARA_FROM_UNDEFINED      12
#define INVALID_PARA_DWG                 21
#define INVALID_PARA_DWG_UNDEFINED       22
#define INVALID_PARA_ID                  31
#define INVALID_PARA_ID_UNDEFINED        32
#define INVALID_PARA_TYPE                41
#define INVALID_PARA_TYPE_UNDEFINED      42
#define INVALID_PARA_SHO                 51
#define INVALID_PARA_SHO_UNDEFINED       52
#define INVALID_PARA_MSG                 61

static FILE *fp = NULL;         /* Log file pointer */

extern void logfput(char*);

/************************************************************************
*                                                                       *
*   Open the log file                                                   *
*                                                                       *
*       path    : Path Name                                             *
*                                                                       *
************************************************************************/
short logfopen(char *path)
{
    short   retc;
    char    datetime[512];
    SYSTEMTIME st;
    char    msgbuf[MSG_STRING_MAX];

    /*------------------*/
    /* Initial          */
    /*------------------*/
    retc = 0;

    /*------------------*/
    /* Open the file    */
    /*------------------*/
    fp = fopen(path, "a+");
    if (fp == NULL) retc = 1;

    /*------------------*/
    /* Get Date&Time    */
    /*------------------*/
    memset ( datetime, '\0', sizeof(datetime) );
    GetLocalTime ( &st );
    sprintf ( msgbuf, "%04d/%02d/%02d %02d:%02d:%02d   ",
        st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond );
    logfput(msgbuf);

    return(retc);
}
/************************************************************************
*                                                                       *
*   Output message to the console and log file                          *
*                                                                       *
*       buf     : The data for output                                   *
*                                                                       *
************************************************************************/
void logfput(char *buf)
{
    printf("%s", buf);
    if (fp != NULL)
        fprintf(fp, "%s", buf);
}
/************************************************************************
*                                                                       *
*   Close the log file                                                  *
*                                                                       *
************************************************************************/
void logfclse(void)
{
    if (fp != NULL)
        fclose(fp);
    fp = NULL;
}
/************************************************************************
*                                                                       *
*   Output the header message                                           *
*                                                                       *
*       vol     : Volume                                                *
*       grp     : Group                                                 *
*       usr     : User                                                  *
*       dwgid   : DwgID                                                 *
*                                                                       *
************************************************************************/
short loghed(char *vol, char *grp, char *usr, char *dwgid)
{
    int     ofst;
    int     len;
    char    msgbuf[MSG_STRING_MAX];
    char    msgbuf1[MSG_STRING_MAX];

    /*------------------*/
    /* Initial          */
    /*------------------*/
    memset(msgbuf, '\0', sizeof(msgbuf));

    /*---------------------------------------------------*/
    /* Make the header message (vol, group, user, dwgid) */
    /*---------------------------------------------------*/
    ofst = (int)strlen(msgbuf);
    len = VOL_LEN;
    memcpy(&msgbuf[ofst], vol, len);
    ofst += len;
    msgbuf[ofst] = SPACE_CHAR;
    ofst++;
    len = GROUP_LEN;
    memcpy(&msgbuf[ofst], grp, len);
    ofst += len;
    msgbuf[ofst] = SPACE_CHAR;
    ofst++;
    len = USER_LEN;
    memcpy(&msgbuf[ofst], usr, len);
    ofst += len;
    msgbuf[ofst] = SPACE_CHAR;
    ofst++;
    msgbuf[ofst] = '[';
    ofst++;
    len = DWGID_LEN;
    memcpy(&msgbuf[ofst], dwgid, len);
    ofst += len;
    msgbuf[ofst] = ']';
    sprintf(msgbuf1, "%s   ", msgbuf);

    /*------------------------------*/
    /* Output vol, group and user   */
    /*------------------------------*/
    logfput(msgbuf1);
    return(0);
}
/************************************************************************
*                                                                       *
*   Output the drawing name and error code                              *
*                                                                       *
*       errcode : Error Code                                            *
*                                                                       *
************************************************************************/
short logdwgid(long errcode)
{
    char    msgbuf[MSG_STRING_MAX];

    /*--------------*/
    /* Initial      */
    /*--------------*/
    memset(msgbuf, '\0', sizeof(msgbuf));
    /*--------------------------------------*/
    /* Output the drawing name and message  */
    /*--------------------------------------*/
    switch(errcode) {
    case 0:
        strcpy(msgbuf, "Complete");
        break;
    case 1:
        strcpy(msgbuf, "Error  Other Error");
        break;
    case 4:
        strcpy(msgbuf, "Error  Invalid Volume");
        break;
    case 5:
        strcpy(msgbuf, "Error  Drawing Too Big");
        break;
    case 11:
        strcpy(msgbuf, "Error  Disk Full");
        break;
    case 13:
        strcpy(msgbuf, "Error  Group, User Not Found");
        break;
    case 16:
        strcpy(msgbuf, "Error  I/O Error");
        break;
    case 18:
        strcpy(msgbuf, "Error  Drawing Not Found");
        break;
    case 20:
        strcpy(msgbuf, "Error  RO Drawing");
        break;
    case 21:
        strcpy(msgbuf, "Error  Incompatible Drawing");
        break;
    case 23:
        strcpy(msgbuf, "Error  PW Drawing");
        break;
    case 24:
        strcpy(msgbuf, "Error  Bad Drawing");
        break;
    case 25:
        strcpy(msgbuf, "Error  Volume Not Supported");
        break;
    case 26:
        strcpy(msgbuf, "Error  Volume Not Exits");
        break;
    case 27:
        strcpy(msgbuf, "Error  Already Renewed");
        break;
    case 31:
        strcpy(msgbuf, "Error  Same Drawing Has Opened");
        break;
    case 37:
        strcpy(msgbuf, "Error  SC Drawing");
        break;
    case 72:
        strcpy(msgbuf, "Error  Group, User Not Exit in Privilege");
        break;
    case 73:
        strcpy(msgbuf, "Error  Permission Denied");
        break;
    case 100:
        strcpy(msgbuf, "Error  Drawing Number Exceeded");
        break;
    case 200:
        strcpy(msgbuf, "Error  Out of Memory");
        break;
    case 400:
        strcpy(msgbuf, "Error  No Active Member");
        break;
    case 1001:
        strcpy(msgbuf, "Error  V2 Drawing");
        break;
    case 1002:
        strcpy(msgbuf, "Error  LK Drawing");
        break;
    default:
        strcpy(msgbuf, "Error  Other Error");
        break;
    } /* end of switch */
    strcat(msgbuf, "\n");
    logfput(msgbuf);
    return(0);
}
/************************************************************************
*                                                                       *
*   Output the parameter error message                                  *
*                                                                       *
*       errcode : Error Code                                            *
*                                                                       *
************************************************************************/
short logmsg(long errcode)
{
    char    msgbuf[MSG_STRING_MAX];
    char    msgbuf1[MSG_STRING_MAX];

    /*--------------*/
    /* Initial      */
    /*--------------*/
    memset(msgbuf, '\0', sizeof(msgbuf));
    /*--------------------------------------*/
    /* Output the parameter error message   */
    /*--------------------------------------*/
    switch(errcode) {
    case 0:
        strcpy(msgbuf, "Complete\n");
        logfput(msgbuf);
        break;
    case INVALID_PARA:
        strcpy(msgbuf, "\nParameter input error\n");
        logfput(msgbuf);
        strcpy(msgbuf, "\ne.g.   ( )-Abbreviation\n");
        logfput(msgbuf);
        strcpy(msgbuf, "FROM(F)=volume:group,user\n");
        logfput(msgbuf);
        strcpy(msgbuf, "DWG(D)=drawing name\n");
        logfput(msgbuf);
        strcpy(msgbuf, "ID(I)=userid,password\n");
        logfput(msgbuf);
        strcpy(msgbuf, "TYPE(T)=A/100\n");
        logfput(msgbuf);
        strcpy(msgbuf, "SHO(S)=A/0\n");
        logfput(msgbuf);
        strcpy(msgbuf, "[MSG(M)=msg log file name]\n");
        logfput(msgbuf);
        break;
    case INVALID_PARA_FROM:
        strcpy(msgbuf1, "FROM");
        break;
    case INVALID_PARA_FROM_UNDEFINED:
        strcpy(msgbuf1, "FROM UNDEFINED");
        break;
    case INVALID_PARA_DWG           :
        strcpy(msgbuf1, "DWG");
        break;
    case INVALID_PARA_DWG_UNDEFINED :
        strcpy(msgbuf1, "DWG UNDEFINED");
        break;
    case INVALID_PARA_ID            :
        strcpy(msgbuf1, "ID");
        break;
    case INVALID_PARA_ID_UNDEFINED  :
        strcpy(msgbuf1, "ID UNDEFINED");
        break;
    case INVALID_PARA_TYPE          :
        strcpy(msgbuf1, "TYPE A/100");
        break;
    case INVALID_PARA_TYPE_UNDEFINED:
        strcpy(msgbuf1, "TYPE UNDEFINED");
        break;
    case INVALID_PARA_SHO           :
        strcpy(msgbuf1, "SHO A/0");
        break;
    case INVALID_PARA_SHO_UNDEFINED :
        strcpy(msgbuf1, "SHO UNDEFINED");
        break;
    case INVALID_PARA_MSG           :
        strcpy(msgbuf1, "MSG");
        break;
    default:
        strcpy(msgbuf, "Error  Other Error\n");
        logfput(msgbuf);
        break;
    } /* end of switch */

    if( errcode >= INVALID_PARA_FROM && errcode <= INVALID_PARA_MSG ) {
        sprintf(msgbuf, "Parameter error [%s]\n", msgbuf1);
        logfput(msgbuf);
    }

    return(0);
}
